# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: arithmetic/subtract.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x19\x61rithmetic/subtract.proto\x12\nArithmetic\"\x19\n\x07Minuend\x12\x0e\n\x06number\x18\x01 \x01(\x05\"\x1c\n\nDifference\x12\x0e\n\x06number\x18\x01 \x01(\x05\x32I\n\x0bSubtraction\x12:\n\x0bSubtractOne\x12\x13.Arithmetic.Minuend\x1a\x16.Arithmetic.Differenceb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'arithmetic.subtract_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _globals['_MINUEND']._serialized_start=41
  _globals['_MINUEND']._serialized_end=66
  _globals['_DIFFERENCE']._serialized_start=68
  _globals['_DIFFERENCE']._serialized_end=96
  _globals['_SUBTRACTION']._serialized_start=98
  _globals['_SUBTRACTION']._serialized_end=171
# @@protoc_insertion_point(module_scope)
