# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: Testing_protos/AddAltTypes.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n Testing_protos/AddAltTypes.proto\x12\x07Testing\"\x18\n\x06\x41\x64\x64\x65nd\x12\x0e\n\x06number\x18\x01 \x01(\x05\"\x15\n\x03Sum\x12\x0e\n\x06number\x18\x01 \x01(\x05\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'Testing_protos.AddAltTypes_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _globals['_ADDEND']._serialized_start=45
  _globals['_ADDEND']._serialized_end=69
  _globals['_SUM']._serialized_start=71
  _globals['_SUM']._serialized_end=92
# @@protoc_insertion_point(module_scope)
