# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: Testing_protos/AddTypes.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from Testing_protos import AddAltTypes_pb2 as Testing__protos_dot_AddAltTypes__pb2

from Testing_protos.AddAltTypes_pb2 import *

DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1dTesting_protos/AddTypes.proto\x12\x07Testing\x1a Testing_protos/AddAltTypes.proto\"\x84\x01\n\x0bStatusQuery\x12,\n\x05\x63ount\x18\xe9\x07 \x01(\x0b\x32\x1a.Testing.StatusQuery.CountH\x00\x12,\n\x05state\x18\xea\x07 \x01(\x0b\x32\x1a.Testing.StatusQuery.StateH\x00\x1a\x07\n\x05\x43ount\x1a\x07\n\x05StateB\x07\n\x05query\"\xe3\x02\n\x0eStatusResponse\x12/\n\x05\x63ount\x18\xd1\x0f \x01(\x0b\x32\x1d.Testing.StatusResponse.CountH\x00\x12/\n\x05state\x18\xd2\x0f \x01(\x0b\x32\x1d.Testing.StatusResponse.StateH\x00\x1a\x17\n\x05\x43ount\x12\x0e\n\x06number\x18\x01 \x01(\r\x1a\xc9\x01\n\x05State\x12\x39\n\x06states\x18\x01 \x03(\x0b\x32).Testing.StatusResponse.State.StatesEntry\x1a/\n\x07Wrapper\x12$\n\x05state\x18\x01 \x01(\x0e\x32\x15.Testing.ServiceState\x1aT\n\x0bStatesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x34\n\x05value\x18\x02 \x01(\x0b\x32%.Testing.StatusResponse.State.Wrapper:\x02\x38\x01\x42\n\n\x08response*?\n\x0cServiceState\x12\x0c\n\x08_UNKNOWN\x10\x00\x12\x16\n\x12\x42USY_COMPUTING_ONE\x10\x01\x12\t\n\x05READY\x10\x02P\x00\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'Testing_protos.AddTypes_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _STATUSRESPONSE_STATE_STATESENTRY._options = None
  _STATUSRESPONSE_STATE_STATESENTRY._serialized_options = b'8\001'
  _globals['_SERVICESTATE']._serialized_start=569
  _globals['_SERVICESTATE']._serialized_end=632
  _globals['_STATUSQUERY']._serialized_start=77
  _globals['_STATUSQUERY']._serialized_end=209
  _globals['_STATUSQUERY_COUNT']._serialized_start=184
  _globals['_STATUSQUERY_COUNT']._serialized_end=191
  _globals['_STATUSQUERY_STATE']._serialized_start=193
  _globals['_STATUSQUERY_STATE']._serialized_end=200
  _globals['_STATUSRESPONSE']._serialized_start=212
  _globals['_STATUSRESPONSE']._serialized_end=567
  _globals['_STATUSRESPONSE_COUNT']._serialized_start=328
  _globals['_STATUSRESPONSE_COUNT']._serialized_end=351
  _globals['_STATUSRESPONSE_STATE']._serialized_start=354
  _globals['_STATUSRESPONSE_STATE']._serialized_end=555
  _globals['_STATUSRESPONSE_STATE_WRAPPER']._serialized_start=422
  _globals['_STATUSRESPONSE_STATE_WRAPPER']._serialized_end=469
  _globals['_STATUSRESPONSE_STATE_STATESENTRY']._serialized_start=471
  _globals['_STATUSRESPONSE_STATE_STATESENTRY']._serialized_end=555
# @@protoc_insertion_point(module_scope)
